# PLEASE NOTE that the PSCX Module is required for the Write-ZIP function.
# The modules can be found here
# http://pscx.codeplex.com

Import-Module PSCX
Write-Output "Building RobotinoMRDS ZIP package"

#################################################################

# Create the top-level directories
# NOTE: Normally Copy-Item should create these directories if they
# don't exist but there seems to be an issue of the files not being
# copied properly

New-Item -Path $Env:TEMP\RobotinoMRDS -Name Bin -ItemType Directory
New-Item -Path $Env:TEMP\RobotinoMRDS -Name Examples -ItemType Directory
New-Item -Path $Env:TEMP\RobotinoMRDS -Name Services -ItemType Directory
New-Item -Path $Env:TEMP\RobotinoMRDS -Name Simulation -ItemType Directory
New-Item -Path $Env:TEMP\RobotinoMRDS -Name Store -ItemType Directory


# Create a RobotinoMRDS folder in the TEMP dir 
# and copy the relevant files there

foreach ($myfile in Get-ChildItem -Exclude ".svn")
{
	Copy-Item $myfile -Destination $Env:TEMP\RobotinoMRDS -Recurse
}

# Now copy the following DLLs using relevant env variables
# into the Bin folder.
# rec_robotino_api2_dotnet.dll
# rec_robotino_api2_dotnet_wrapper.dll
# rec_robotino_api2.dll

Copy-Item $Env:ROBOTINOAPI2WRAPPER32_DIR\bin\rec_robotino_api2_dotnet.dll -Destination $Env:TEMP\RobotinoMRDS\Bin
Copy-Item $Env:ROBOTINOAPI2WRAPPER32_DIR\bin\rec_robotino_api2_dotnet_wrapper.dll -Destination $Env:TEMP\RobotinoMRDS\Bin
Copy-Item $Env:ROBOTINOAPI2_32_DIR\bin\rec_robotino_api2.dll -Destination $Env:TEMP\RobotinoMRDS\Bin

#################################################################
# Modify the Reference path and PreBuildEvent properties of the
# Robotino.csproj file. 

$path = "$Env:TEMP\RobotinoMRDS\Services\Robotino\Robotino.csproj"
$xml = [xml](Get-Content $path)
$reference = $xml.Project.ItemGroup[0].Reference | Where { $_.Include.startsWith("rec") }
$xml.Project.PropertyGroup[4].PreBuildEvent = ""
$reference.HintPath = "..\..\..\..\bin\rec_robotino_api2_dotnet.dll"
$xml.Save($path)

#################################################################

# Zip everything up and copy the zipped folder to the current dir

$dt = Get-Date
$dtstr = $dt.Year.ToString() + $dt.Month.ToString() + $dt.Day.ToString()
Write-ZIP $Env:TEMP\RobotinoMRDS ("RobotinoMRDS_" + $dtstr + ".zip")

#################################################################
# Clean up

Remove-Item -Recurse -Force $Env:TEMP\RobotinoMRDS

#################################################################

# Done!
Write-Host "Don't worry about errors above. Operation complete! Press any key to continue ...";

$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown");

#################################################################