using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

#region Additional namespaces
using robotino = RobotinoMRDS.Services.Robotino.Proxy;
using irsensorarray = Microsoft.Robotics.Services.AnalogSensorArray.Proxy;
using xna = Microsoft.Xna.Framework;
using System.Diagnostics;
#endregion

namespace RobotinoMRDS.Examples.DriveCircle
{
    [Contract(Contract.Identifier)]
    [DisplayName("DriveCircle")]
    [Description("A service that drives Robotino in a circle")]
    class DriveCircleService : DsspServiceBase
    {
        // Used to communicate with Robotino's OmniDrive
        [Partner(
            "Robotino",
            Contract = robotino.Contract.Identifier,
            CreationPolicy = PartnerCreationPolicy.UseExisting,
            Optional = false)]
        robotino.RobotinoOperations _robotinoPort = new robotino.RobotinoOperations();

        /// <summary>
        /// Service state
        /// </summary>
        [ServiceState]
        DriveCircleState _state = new DriveCircleState();

        /// <summary>
        /// Main service port
        /// </summary>
        [ServicePort("/DriveCircle", AllowMultipleInstances = true)]
        DriveCircleOperations _mainPort = new DriveCircleOperations();

        /// <summary>
        /// Service constructor
        /// </summary>
        public DriveCircleService(DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
        }

        /// <summary>
        /// Service start
        /// </summary>
        protected override void Start()
        {
            base.Start();
            SpawnIterator<DateTime>(DateTime.Now, Drive);
        }

        IEnumerator<ITask> Drive(DateTime timeout)
        {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();

            float[] startVector = new float[] { 0.2f, 0.0f};
            float[] dir = new float[2];
            float a = 0.0f;

            while (true)
            {
                rotate(startVector, dir, a);
                a = 360.0f * stopwatch.ElapsedMilliseconds / 5000;
                Activate(Arbiter.Choice(
                    _robotinoPort.OmniDrive((float)dir[0], (float)dir[1], 0),
                    delegate(DefaultUpdateResponseType response) { },
                    delegate(Fault f) { LogError(f); }
                ));

                yield return Arbiter.Receive(false, TimeoutPort(100), delegate { });
            }
        }

        void rotate(float[] inArray, float[] outArray, float a)
        {
            float rad = xna.MathHelper.ToRadians(a);
            outArray[0] = (float)Math.Cos(rad) * inArray[0] - (float)Math.Sin(rad) * inArray[1];
            outArray[1] = (float)Math.Sin(rad) * inArray[0] + (float)Math.Cos(rad) * inArray[1];
        }
    }
}