﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

#region Simulation namespaces
using Microsoft.Robotics.Simulation;
using Microsoft.Robotics.Simulation.Engine;
using Microsoft.Robotics.Simulation.Physics;
using Microsoft.Robotics.PhysicalModel;
using xna = Microsoft.Xna.Framework;
using xnagrfx = Microsoft.Xna.Framework.Graphics;
#endregion

namespace RobotinoMRDS.Simulation.Robotino
{
    [DataContract]
    public class RobotinoIRArrayEntity : VisualEntity
    {
        [DataMember]
        RobotinoIREntity[] _irSensors;

        [DataMember]
        public RobotinoIREntity[] IRSensors
        {
            get { return _irSensors; }
            set { _irSensors = value; }
        }
        const int NO_OF_IR_SENSORS = 9;

        // Default Constructor
        public RobotinoIRArrayEntity()
        {
        }

        /// Initialization constructor used when this entity is built programmatically
        /// </summary>
        /// <param name="initialPos"></param>
        public RobotinoIRArrayEntity(string name, Pose pose)
        {
            base.State.Name = name;
            base.State.Pose = pose;
        }

        public override void Initialize(xnagrfx.GraphicsDevice device, PhysicsEngine physicsEngine)
        {
            try
            {
                InitError = string.Empty;

                _irSensors = new RobotinoIREntity[NO_OF_IR_SENSORS];

                string[] irSensorName = new string[] { "IR1", "IR2", "IR3", 
                    "IR4", "IR5", "IR6", 
                    "IR7", "IR8", "IR9"};

                float start = (float)(-Math.PI);
                float end = (float)(Math.PI);
                float increment = (end - start) / (NO_OF_IR_SENSORS);
                int segment = 0;


                for (float angle = start; angle <= end; angle += increment, segment++)
                {
                    RobotinoIREntity irSensor = new RobotinoIREntity(
                            irSensorName[segment],
                            new Pose(new Vector3(0.16f * (float)(Math.Sin(angle)), 0.01f,
                                0.16f * (float)(Math.Cos(angle))),
                                Quaternion.FromAxisAngle(0, 1, 0, angle)));

                    irSensor.Parent = this;
                    _irSensors[segment] = irSensor;
                }

                //_irSensors[0] = new RobotinoIREntity("IR1",
                //   new Pose(new Vector3(0.0f, 0.0f, -0.2f),
                //       Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(180))));
                //_irSensors[0].Parent = this;

                //_irSensors[1] = new RobotinoIREntity("IR2",
                //    new Pose(new Vector3(-0.129f, 0.0f, -0.153f),
                //    Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(-140))));
                //_irSensors[1].Parent = this;

                //_irSensors[2] = new RobotinoIREntity("IR3",
                //    new Pose(new Vector3(-0.197f, 0.0f, -0.035f),
                //    Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(-100))));
                //_irSensors[2].Parent = this;

                //_irSensors[3] = new RobotinoIREntity("IR4",
                //    new Pose(new Vector3(-0.173f, 0.0f, 0.1f),
                //    Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(-60))));
                //_irSensors[3].Parent = this;

                //_irSensors[4] = new RobotinoIREntity("IR5",
                //    new Pose(new Vector3(-0.068f, 0.0f, 0.188f),
                //    Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(-20))));
                //_irSensors[4].Parent = this;

                //_irSensors[5] = new RobotinoIREntity("IR6",
                //    new Pose(new Vector3(0.068f, 0.0f, 0.188f),
                //    Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(20))));
                //_irSensors[5].Parent = this;

                //_irSensors[6] = new RobotinoIREntity("IR7",
                //    new Pose(new Vector3(0.173f, 0.0f, 0.1f),
                //    Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(60))));
                //_irSensors[6].Parent = this;

                //_irSensors[7] = new RobotinoIREntity("IR8",
                //    new Pose(new Vector3(0.197f, 0.0f, -0.035f),
                //    Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(100))));
                //_irSensors[7].Parent = this;

                //_irSensors[8] = new RobotinoIREntity("IR9",
                //    new Pose(new Vector3(0.129f, 0.0f, -0.153f),
                //    Quaternion.FromAxisAngle(0, 1, 0, xna.MathHelper.ToRadians(140))));
                //_irSensors[8].Parent = this;

                foreach (RobotinoIREntity ir in _irSensors)
                {
                    InsertEntity(ir);
                }

                base.Initialize(device, physicsEngine);

                if (Parent == null)
                    throw new Exception("This entity must be a child of another entity.");

            }
            catch (Exception ex)
            {
                HasBeenInitialized = false;
                InitError = ex.ToString();
            }
        }
    }

    #region Single IR Entity
    /// <summary>
    /// An IR distance sensor.
    /// </summary>
    [DataContract]
    public class RobotinoIREntity : VisualEntity
    {
        [DataMember]
        public float DispersionConeAngle = 3f; // in degrees
        [DataMember]
        public float Samples = 1f;  // the number of rays in each direction
        [DataMember]
        public float MaximumRange = (40f / 100.0f); // 40 cm converted to meters

        const float SCAN_INTERVAL = 0.2f;

        float _elapsedSinceLastScan;
        Port<RaycastResult> _raycastResultsPort;
        RaycastResult _lastResults;
        RaycastProperties _raycastProperties;
        CachedEffectParameter _timeAttenuationHandle;
        float _appTime;
        Shape _particlePlane;

        /// <summary>
        /// Raycast configuration
        /// </summary>
        public RaycastProperties RaycastProperties
        {
            get { return _raycastProperties; }
            set { _raycastProperties = value; }
        }

        float _distance;
        [DataMember]
        public float Distance
        {
            get { return _distance; }
            set { _distance = value; }
        }

        /// <summary>
        /// Default constructor used when this entity is deserialized
        /// </summary>
        public RobotinoIREntity()
        {
        }

        /// <summary>
        /// Initialization constructor used when this entity is built programmatically
        /// </summary>
        /// <param name="initialPos"></param>
        public RobotinoIREntity(string name, Pose initialPose)
        {
            base.State.Name = name;
            base.State.Pose = initialPose;
            // used for rendering impact points
            base.State.Assets.Effect = "LaserRangeFinder.fx";
        }

        public override void Initialize(xnagrfx.GraphicsDevice device, PhysicsEngine physicsEngine)
        {
            try
            {
                InitError = string.Empty;

                if (Parent == null)
                    throw new Exception("This entity must be a child of another entity.");

                _raycastProperties = new RaycastProperties();
                _raycastProperties.StartAngle = -DispersionConeAngle / 2.0f;
                _raycastProperties.EndAngle = DispersionConeAngle / 2.0f;
                _raycastProperties.AngleIncrement = DispersionConeAngle;
                _raycastProperties.Range = MaximumRange;
                _raycastProperties.OriginPose = new Pose();

                // set flag so rendering engine renders us last
                Flags |= VisualEntityProperties.UsesAlphaBlending;

                base.Initialize(device, physicsEngine);

                // set up for rendering impact points
                HeightFieldShapeProperties hf = new HeightFieldShapeProperties("height field", 2, 0.01f, 2, 0.01f, 0, 0, 1, 1);
                hf.HeightSamples = new HeightFieldSample[hf.RowCount * hf.ColumnCount];
                for (int i = 0; i < hf.HeightSamples.Length; i++)
                    hf.HeightSamples[i] = new HeightFieldSample();

                _particlePlane = new Shape(hf);
                _particlePlane.State.Name = "infrared impact plane";

                // The mesh is used to render the ray impact points rather than the sensor geometry.
                int index = Meshes.Count;
                Meshes.Add(SimulationEngine.ResourceCache.CreateMesh(device, _particlePlane.State));
                Meshes[0].Textures[0] = SimulationEngine.ResourceCache.CreateTextureFromFile(device, "particle.bmp");

                // we have a custom effect, with an additional global parameter. Get handle to it here
                if (Effect != null)
                    _timeAttenuationHandle = Effect.GetParameter("timeAttenuation");
            }
            catch (Exception ex)
            {
                // clean up
                if (PhysicsEntity != null)
                    PhysicsEngine.DeleteEntity(PhysicsEntity);

                HasBeenInitialized = false;
                InitError = ex.ToString();
            }
        }


        public override void Update(FrameUpdate update)
        {
            base.Update(update);
            _elapsedSinceLastScan += (float)update.ElapsedTime;
            _appTime = (float)update.ApplicationTime;

            // only retrieve raycast results every SCAN_INTERVAL.
            // For entities that are compute intenisve, you should consider giving them
            // their own task queue so they dont flood a shared queue
            if ((_elapsedSinceLastScan > SCAN_INTERVAL) && (_raycastProperties != null))
            {
                _elapsedSinceLastScan = 0;

                _raycastProperties.OriginPose.Orientation = TypeConversion.FromXNA(
                    TypeConversion.ToXNA(Parent.Parent.State.Pose.Orientation) *
                    TypeConversion.ToXNA(State.Pose.Orientation));

                _raycastProperties.OriginPose.Position = TypeConversion.FromXNA(
                    xna.Vector3.Transform(TypeConversion.ToXNA(State.Pose.Position), Parent.Parent.World));

                // cast rays on a horizontal plane
                _raycastResultsPort = PhysicsEngine.Raycast2D(_raycastProperties);
                _raycastResultsPort.Test(out _lastResults);

                // find the shortest distance to an impact point
                float minRange = MaximumRange * MaximumRange;
                xna.Vector4 origin = new xna.Vector4(TypeConversion.ToXNA(_raycastProperties.OriginPose.Position), 1);
                foreach (RaycastImpactPoint impact in _lastResults.ImpactPoints)
                {
                    xna.Vector3 impactVector = new xna.Vector3(
                        impact.Position.X - origin.X,
                        impact.Position.Y - origin.Y,
                        impact.Position.Z - origin.Z);

                    float impactDistanceSquared = impactVector.LengthSquared();
                    if (impactDistanceSquared < minRange)
                        minRange = impactDistanceSquared;
                }
                _distance = (float)Math.Sqrt(minRange);
            }
        }
        /// <summary>
        /// Frame render
        /// </summary>
        public override void Render(RenderMode renderMode, MatrixTransforms transforms, CameraEntity currentCamera)
        {
            if ((int)(Flags & VisualEntityProperties.DisableRendering) > 0)
                return;

            if (_lastResults != null)
                RenderResults(renderMode, transforms, currentCamera);
        }

        void RenderResults(RenderMode renderMode, MatrixTransforms transforms, CameraEntity currentCamera)
        {
            _timeAttenuationHandle.SetValue(new xna.Vector4(100 * (float)Math.Cos(_appTime * (1.0f / SCAN_INTERVAL)), 0, 0, 1));

            // render impact points as a quad
            xna.Matrix inverseViewRotation = currentCamera.ViewMatrix;
            inverseViewRotation.M41 = inverseViewRotation.M42 = inverseViewRotation.M43 = 0;
            xna.Matrix.Invert(ref inverseViewRotation, out inverseViewRotation);
            xna.Matrix localTransform = xna.Matrix.CreateFromAxisAngle(new xna.Vector3(1, 0, 0), (float)-Math.PI / 2) * inverseViewRotation;

            //SimulationEngine.GlobalInstance.Device.DepthStencilState.DepthBufferWriteEnable = false;

            for (int i = 0; i < _lastResults.ImpactPoints.Count; i++)
            {
                xna.Vector3 pos = new xna.Vector3(_lastResults.ImpactPoints[i].Position.X,
                    _lastResults.ImpactPoints[i].Position.Y,
                    _lastResults.ImpactPoints[i].Position.Z);

                xna.Vector3 resultDir = pos - Parent.Position;
                resultDir.Normalize();

                localTransform.Translation = pos - .02f * resultDir;
                transforms.World = localTransform;

                base.Render(renderMode, transforms, Meshes[0]);
            }
            //SimulationEngine.GlobalInstance.Device.DepthStencilState.DepthBufferWriteEnable = true;
        }
    }
    #endregion
}
