using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

#region Simulation namespaces
using Microsoft.Robotics.Simulation;
using Microsoft.Robotics.Simulation.Engine;
using engineproxy = Microsoft.Robotics.Simulation.Engine.Proxy;
using Microsoft.Robotics.Simulation.Physics;
using Microsoft.Robotics.PhysicalModel;
using xna = Microsoft.Xna.Framework;
#endregion

namespace RobotinoMRDS.Simulation.Robotino
{
    [Contract(Contract.Identifier)]
    [DisplayName("RobotinoTest")]
    [Description("A simple service to test Robotino")]
    class RobotinoTestService : DsspServiceBase
    {
        /// <summary>
        /// Service state
        /// </summary>
        [ServiceState]
        RobotinoTestState _state = new RobotinoTestState();

        /// <summary>
        /// Main service port
        /// </summary>
        [ServicePort("/RobotinoTest", AllowMultipleInstances = true)]
        RobotinoTestOperations _mainPort = new RobotinoTestOperations();

        /// <summary>
        /// SimulationEngine partner
        /// </summary>
        [Partner("Engine", Contract = engineproxy.Contract.Identifier, CreationPolicy = PartnerCreationPolicy.UseExistingOrCreate)]
        private engineproxy.SimulationEnginePort _engineStub = new engineproxy.SimulationEnginePort();

        /// <summary>
        /// Service constructor
        /// </summary>
        public RobotinoTestService(DsspServiceCreationPort creationPort)
            : base(creationPort)
        {
        }

        /// <summary>
        /// Service start
        /// </summary>
        protected override void Start()
        {
            base.Start();

            // Create a very simple world and add Robotino to it

            // MainCamera initial view
            CameraView view = new CameraView();
            view.EyePosition = new Vector3(-1.65f, 1.63f, -0.29f);
            view.LookAtPoint = new Vector3(0, 0, 0);
            SimulationEngine.GlobalInstancePort.Update(view);

            // Add a SkyDome.
            SkyDomeEntity sky = new SkyDomeEntity("skydome.dds", "sky_diff.dds");
            SimulationEngine.GlobalInstancePort.Insert(sky);

            // Add a directional light to simulate the sun.
            LightSourceEntity sun = new LightSourceEntity();
            sun.State.Name = "Sun";
            sun.Type = LightSourceEntityType.Directional;
            sun.Color = new Vector4(0.8f, 0.8f, 0.8f, 1);
            sun.Direction = new Vector3(0.5f, -.75f, 0.5f);
            SimulationEngine.GlobalInstancePort.Insert(sun);

            // create a large horizontal plane, at zero elevation.
            HeightFieldEntity ground = new HeightFieldEntity(
                "Ground", // name
                "concrete_Mayang_b.dds", // texture image
                new MaterialProperties("ground",
                    0.2f, // restitution
                    0.5f, // dynamic friction
                    0.5f) // static friction
                );
            SimulationEngine.GlobalInstancePort.Insert(ground);

            Vector3 dimensions = new Vector3(2f, 2f, 2f); // meters

            // create simple movable entity, with a single shape
            SingleShapeEntity box = new SingleShapeEntity(
                new BoxShape(
                    new BoxShapeProperties(
                    100, // mass in kilograms.
                    new Pose(), // relative pose
                    dimensions)), // dimensions
                    new Vector3(0, 1f, 29f * 2.54f / 100f + 1f));

            // Name the entity. All entities must have unique names
            box.State.Name = "BigBox";

            // Insert entity in simulation.  
            SimulationEngine.GlobalInstancePort.Insert(box);

            //Insert Robotino
            SimulationEngine.GlobalInstancePort.Insert(
                new RobotinoEntity("Robotino", new Pose()));
        }
    }
}


