using System;
using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Ccr.Core;
using Microsoft.Dss.Core.Attributes;
using Microsoft.Dss.ServiceModel.Dssp;
using Microsoft.Dss.ServiceModel.DsspServiceBase;
using W3C.Soap;

#region Additional Namespaces
using drive = Microsoft.Robotics.Services.Drive.Proxy;
using motor = Microsoft.Robotics.Services.Motor.Proxy;
using Microsoft.Robotics.PhysicalModel;
using Microsoft.Dss.Core.DsspHttp;
#endregion

namespace RobotinoMRDS.Simulation.RobotinoSimulatedRobotinoDrive
{
    /// <summary>
    /// SimulatedRobotinoDrive contract class
    /// </summary>
    public sealed class Contract
    {
        /// <summary>
        /// DSS contract identifer for SimulatedRobotinoDrive
        /// </summary>
        [DataMember]
        public const string Identifier = "http://servicerobotics.eu/2011/11/simulatedrobotinodrive.html";
    }

    /// <summary>
    /// SimulatedRobotinoDrive state
    /// </summary>
    [DataContract]
    public class SimulatedRobotinoDriveState : drive.DriveDifferentialTwoWheelState
    {
        private motor.WheeledMotorState _wheel1;
        private Vector3 _position;

        [Description("The rear wheel's state")]
        [DataMember]
        public motor.WheeledMotorState Wheel1
        {
            get { return _wheel1; }
            set { _wheel1 = value; }
        }

        [Description("The current position of the entity")]
        [DataMember]
        public Vector3 Position
        {
            get { return _position; }
            set { _position = value; }
        }
    }

    /// <summary>
    /// SimulatedRobotinoDrive main operations port
    /// </summary>
    [ServicePort]
    public class SimulatedRobotinoDriveOperations : PortSet<
        DsspDefaultLookup, 
        DsspDefaultDrop, 
        Get,
        HttpGet,
        Subscribe, 
        ReliableSubscribe, 
        SetPose, 
        OmniDrive, 
        Rotate>
    {
    }

    /// <summary>
    /// SimulatedRobotinoDrive get operation
    /// </summary>
    public class Get : Get<GetRequestType, PortSet<SimulatedRobotinoDriveState, Fault>>
    {
    }

    /// <summary>
    /// SimulatedRobotinoDrive subscribe operation
    /// </summary>
    public class Subscribe : Subscribe<SubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        public Subscribe()
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        public Subscribe(SubscribeRequestType body)
            : base(body)
        {
        }

        /// <summary>
        /// Creates a new instance of Subscribe
        /// </summary>
        /// <param name="body">the request message body</param>
        /// <param name="responsePort">the response port for the request</param>
        public Subscribe(SubscribeRequestType body, PortSet<SubscribeResponseType, Fault> responsePort)
            : base(body, responsePort)
        {
        }
    }
    /// <summary>
    /// Operation to reliable subscribe to drive notifications
    /// </summary>
    public class ReliableSubscribe : Subscribe<ReliableSubscribeRequestType, PortSet<SubscribeResponseType, Fault>>
    {
    }

    /// <summary>
    /// Operation Set Entity Pose
    /// </summary>
    [Description("Sets the pose of the omniDrive entity.")]
    public class SetPose : Update<SetPoseRequestType, PortSet<DefaultUpdateResponseType, Fault>>
    {
    }

    /// <summary>
    /// Operation OmniDrive
    /// </summary>
    [Description("OmniDrive")]
    public class OmniDrive : Update<OmniDriveRequestType, PortSet<DefaultUpdateResponseType, Fault>>
    {
    }

    /// <summary>
    /// Operation Rotate
    /// </summary>
    [Description("Rotate")]
    public class Rotate : Update<RotateRequestType, PortSet<DefaultUpdateResponseType, Fault>>
    {
    }

    /// <summary>
    /// Set entity pose request
    /// </summary>
    [DataMemberConstructor]
    [DataContract]
    public class SetPoseRequestType
    {
        Pose _entityPose;

        [DataMember]
        public Pose EntityPose
        {
            get { return _entityPose; }
            set { _entityPose = value; }
        }

        public SetPoseRequestType()
        {
        }
    }

    /// <summary>
    /// OmniDrive request
    /// </summary>
    [DataMemberConstructor]
    [DataContract]
    public class OmniDriveRequestType
    {
        float _velocityX;
        float _velocityZ;
        float _omega;

        [DataMember]
        public float VelocityX
        {
            get { return _velocityX; }
            set { _velocityX = value; }
        }

        [DataMember]
        public float VelocityZ
        {
            get { return _velocityZ; }
            set { _velocityZ = value; }
        }
        [DataMember]

        public float Omega
        {
            get { return _omega; }
            set { _omega = value; }
        }

        public OmniDriveRequestType()
        {
        }
    }

    /// <summary>
    /// Rotate request
    /// </summary>
    [DataMemberConstructor]
    [DataContract]
    public class RotateRequestType
    {
        int _direction;
        double _power;

        [DataMember]
        public int Direction
        {
            get { return _direction; }
            set { _direction = value; }
        }

        [DataMember]
        public double Power
        {
            get { return _power; }
            set { _power = value; }
        }

        public RotateRequestType()
        {
        }
    }
}


